/*******************************************************************************
$AstecId: treeEd.c,v 1.7 2000/08/25 08:08:54 kosaka Exp $

	treeEd.c

*******************************************************************************/

#ifdef	HASXRTGEAR

#include <stdio.h>

#include <Xm/Xm.h>
#include <Xm/MwmUtil.h>
#include <Xm/DialogS.h>
#include <Xm/MenuShell.h>
#include <Xm/Form.h>
#include <Xm/Separator.h>
#include <Xm/RowColumn.h>
#include <Xm/TextF.h>
#include <Xm/XrtGearAll.h>
#include <Xm/ArrowB.h>
#include <Xm/Form.h>
#include <Xm/FileSB.h>
#include <Xm/Protocols.h>
#include <Xm/AtomMgr.h>

#include <pedit-pop.h>
#include <UxLib.h>
#include <UxDialSh.h>
#include <util.h>
#include "cat_macros.h"
/*******************************************************************************
       Includes, Defines, and Global variables from the Declarations Editor:
*******************************************************************************/


#define OPEN_FILE    0
#define SAVE_FILE    1
#define SAVE_FILE_AS 2

#define	RES_CONVERT( res_name, res_value) \
	XtVaTypedArg, (res_name), XmRString, (res_value), strlen(res_value) + 1

static String helpMsg[] = {
"XRT Tree Editor",
"---------------",
"",
"It allows you to:",
" *  Open and save tree data files",
" *  Apply selected data file as xrtGearTreeData resource value in UIM/X",
" *  Create and edit tree data",
" *  View multicolumn data",
"",
"To create an item in the tree:",
" *  Select item to create (Node/Folder).",
" *  Set label. (Any text displayed in the text field at creation time",
"    will be aplied as a label.)",
" *  Press Add Item to complete procedure.",
"    Note: To create an item inside selected folder, that folder must be opened.",
"    Otherwise, item is added to the end of the tree.",
"",
"To set label on an item:",
" *  Select item in the tree.",
" *  This will display label value in the text field.",
" *  Multicolumn labels are displayed as list of strings separated by ','.",
" *  Edit label value and press Return to apply changes.",
" *  There is also 'View Multi-column' option provided for your convenience.",
NULL
};


void modifyLabelCB();
void createNodeCB();
void deleteNodeCB();
void toggleNodeOrFolderCB();
void popupFSB();
void FSB_cancelCB();
void FSB_okCB();
void openTreeData();
void selectItemCB();
void nodeMoveCB();
void folderStateCB();
void removeColumn();
void addColumn();
void okCB();
void cancelCB();
void helpCB();
static swidget	create_treeEd();

static Boolean treeEdIsUsed = False;

/* True if the data should be saved before quiting. */
static Boolean	dataModified = False;

static  PeSlotPtr treeEdSlot;
static  swidget  XrtPedit;

static  swidget	treeEd;
static	Widget	form;
static	Widget	XrtToolbar1;
static	Widget	XrtGLabel3;
static	Widget	arrowButton1;
static	Widget	arrowButton2;
static	Widget	XrtOutliner1;
static	Widget	XrtColumn1;
static	Widget	XrtAligner1;
static	Widget	XrtGLabel1;
static	Widget	XrtToggleB1;
static	Widget	XrtGLabel2;
static	Widget	textField1;
static	Widget	rowColumn1;
static	Widget	XrtPushB1;
static	Widget	XrtPushB2;
static	Widget	XrtPushB3;
static	Widget	XrtPushB4;
static	Widget	separator1;
static	Widget	form1;
static	Widget	XrtPushB5;
static	Widget	XrtPushB6;
static	Widget	XrtPushB7;
static	swidget	UxParent;

#ifndef XKLOADDS
#define XKLOADDS
#endif /* XKLOADDS */



/*******************************************************************************
       The following are callback functions.
*******************************************************************************/


/*
 * Get shell of a widget
 */
static Widget
getShell(w)
Widget w;
{
	Widget parent = w;
	while (!XtIsVendorShell(parent) && !XmIsDialogShell(parent)
		   && (w = XtParent(parent)))
		parent = w;
	return (parent);
}

void
createNodeCB(w, client_data, cb)
Widget w;
XtPointer client_data;
XmAnyCallbackStruct *cb;
{
    
    Widget parent, currentNode;
	XrtGearFolderState folderState;
	WidgetClass wclass;
	char *name;
	char *label;
#ifdef __hpux
	int  state;
#endif

	
	if (XrtOutliner1 == NULL) {
		return;
	}
	XtVaGetValues(XrtOutliner1,
				  XmNxrtGearNodeCurrent, &currentNode,
				  NULL);
	if (currentNode == NULL) {
	  parent = XrtOutliner1;
	} else if (XmIsXrtNodeFolder(currentNode) == True) {
	  /* check if folder is opened */
	  XtVaGetValues(currentNode,
					XmNxrtGearFolderState, &folderState,
					NULL);
	  if ( folderState != XRTGEAR_FOLDERSTATE_CLOSED) {
		/* if folder is opened add new item to it */
		parent = currentNode;
	  } else {
		parent = XtParent(currentNode);
	  }
	} else {
	  /* otherwise add new item to its parent */
	  parent = XtParent(currentNode);
	}

#ifdef __hpux
	/* workaround for HP-UX XrtToggleButton problem */
	XtVaGetValues( XrtToggleB1, XmNuserData, (XtPointer*)&state, NULL);
	if( state == 0 ) {
	  name = "folder";
	  wclass = xmXrtNodeFolderObjectClass;
	} else {
	  name = "node";
	  wclass = xmXrtNodeObjectClass;
	}
#else
	if( XrtGearToggleButtonGetState(XrtToggleB1) == 0 ) {
	  name = "folder";
	  wclass = xmXrtNodeFolderObjectClass;
	} else {
	  name = "node";
	  wclass = xmXrtNodeObjectClass;
	}
#endif
	label = XmTextFieldGetString(textField1);

	/* NOTE: Do not pass the NULL label.. it may lead the crash ! */
	XtVaCreateWidget(name, wclass, parent,
					 XmNxrtGearLabel, XrtGearNodeCvtStringToLabel(((label && *label)?label:" "), ","),
					 NULL);

	dataModified = True;

	XtFree(label);
}

void
modifyLabelCB(w, client_data, cb)
Widget w;
XtPointer client_data;
XmAnyCallbackStruct *cb;
{
	XrtGearObject labelObject;  /* either XrtGearString or XrtGearList */
	Widget item;
	char *label;
	Boolean selected = False;
	int n;

	Widget currentNode;

	if (XrtOutliner1 == NULL) {
		return;
	}

	XtVaGetValues(XrtOutliner1,
				  XmNxrtGearNodeCurrent, &currentNode,
				  NULL);

	if (currentNode != NULL) {
	  label = XmTextFieldGetString(textField1);

	  /* NOTE: Do not pass the NULL label.. it may lead the crash ! */

	  labelObject = XrtGearNodeCvtStringToLabel(((label && *label)?label:" "), ",");
	  XtVaSetValues(currentNode,
					XmNxrtGearLabel, labelObject,
					NULL);
	  XtFree(label);

	  dataModified = True;

	  return;

	}
}


void
deleteNodeCB(w, client_data, cb)
Widget w;
XtPointer client_data;
XmAnyCallbackStruct *cb;
{
	Widget currentNode;

	if (XrtOutliner1 == NULL) {
		return;
	}

	XtVaGetValues(XrtOutliner1,
				  XmNxrtGearNodeCurrent, &currentNode,
				  NULL);

	if (currentNode != NULL) {
		XtDestroyWidget(currentNode);

		dataModified = True;
	}
}

#ifdef __hpux
/* workaround for HP-UX XrtToggleButton problem */
void
toggleNodeOrFolderCB(w, client_data, cb)
Widget w;
XtPointer client_data;
XmPushButtonCallbackStruct *cb;
{
	XmString label;
	char	*pixmapfile;
	int	state;

	XtVaGetValues( w, XmNuserData, (XtPointer*)&state, NULL);

	if (cb->reason == XmCR_ACTIVATE) {
	  if(state == 0) {
		label = XmStringCreateSimple("Node");
		pixmapfile = "$XRTHOME/src/icons/item2.xpm";
	  } else {
		label = XmStringCreateSimple("Folder");
		pixmapfile = "$XRTHOME/src/icons/folder_open.xpm";
	  }
	  XtVaSetValues(w,
			RES_CONVERT( XmNxrtGearMask, pixmapfile ),
			RES_CONVERT( XmNxrtGearLabelPixmap, pixmapfile ),
			XmNuserData, (XtPointer)(state ? 0 : -1),
					XmNlabelString, label,
					NULL);	
	  XmStringFree(label);
	}

}
#else
void
toggleNodeOrFolderCB(w, client_data, cb)
Widget w;
XtPointer client_data;
XrtGearStateChangedCallbackStruct *cb;
{
	XmString label;

	if (cb->reason == XRTGEAR_REASON_STATE_END) {
	  if(cb->state == 0) {
		label = XmStringCreateSimple("Folder");
	  } else {
		label = XmStringCreateSimple("Node");
	  }
	  XtVaSetValues(w,
					XmNlabelString, label,
					NULL);	
	  XmStringFree(label);
	}

}
#endif

void
selectItemCB(w, client_data, cb)
Widget w;
XtPointer client_data;
XrtGearSelectCallbackStruct *cb;
{
  char *label;
  
  if (cb->reason == XRTGEAR_REASON_SELECT_BEGIN) {
	label = XrtGearNodeCvtLabelToString( cb->node, ",", NULL);
	XmTextFieldSetString(textField1, label);
  }
}
void
nodeMoveCB(w, client_data, cb)
Widget w;
XtPointer client_data;
XrtGearNodeMoveCallbackStruct *cb;
{
  if (cb->reason == XRTGEAR_REASON_NODE_MOVE_END) {
	dataModified = True;
  }
}
void
folderStateCB(w, client_data, cb)
Widget w;
XtPointer client_data;
XrtGearNodeFolderStateCallbackStruct *cb;
{
  if (cb->reason == XRTGEAR_REASON_FOLDER_CHANGE_END) {
	dataModified = True;
  }
}

void
openTreeData(outliner, dataFile)
Widget outliner;
char *dataFile;
{
  
  if (!dataFile || !*dataFile) {
	return;
  }

  if (outliner == NULL) {
	return;
  }

  XtVaSetValues(outliner,
				XmNxrtGearTreeData, dataFile,
				NULL);
  dataModified = False;
}

static Boolean
saveTreeData(outliner, dataFile)
Widget outliner;
char *dataFile;
{
  FILE *fp;
  Boolean status;

  if (!dataFile || !*dataFile) {
	return False;
  }
  
  if (outliner == NULL) {
	return False;
  }

  if (!(fp = fopen(dataFile, "w"))) {
	printf("Can't open file: <%s>\n", dataFile);
	return False;
  }

  status = XrtGearNodeWriteTreeToStream(outliner, fp, False, False,
										NULL, NULL, NULL, NULL, NULL);  
  fclose(fp);

  if(status == True) {
	/* update name of treeData file "inside" outliner */
	XtVaSetValues(outliner,
				  XmNxrtGearTreeData, dataFile,
				  NULL);
	dataModified = False;
  }else {
	printf("Error in: XrtGearNodeWriteTreeToStream()\n");
  }	

  return status;

}

static void
saveDataFile(outliner, dataFile)
Widget outliner;
char *dataFile;
{
  if(saveTreeData(outliner, dataFile) == True) {
    UxPeDepositValue( XrtPedit, treeEdSlot, UxQuoteWrap(dataFile));
  }
}

/*
 * File Selection Box stuff
 */

void
popupFSB(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer call_data;
{
  int reason = (int) client_data;

  static Widget dialog;
  char *name;

  Atom WM_DELETE_WINDOW;

  /* set FSB dialog title*/
  if (reason == OPEN_FILE) {
	name = "OpenFile";
  } else {
	name = "SaveFile";
  }
  dialog = XmCreateFileSelectionDialog(w, name, NULL, 0);
  XtVaSetValues(getShell(dialog),
				 XmNmwmInputMode, MWM_INPUT_APPLICATION_MODAL,
				 NULL);  
  XtAddCallback(dialog, XmNcancelCallback,
				(XtCallbackProc) FSB_cancelCB, NULL);
  XtAddCallback(dialog, XmNokCallback, FSB_okCB, (XtPointer)reason);
  
  WM_DELETE_WINDOW = XmInternAtom(XtDisplay(dialog),
								  "WM_DELETE_WINDOW", False);
  XmAddWMProtocolCallback(getShell(dialog), WM_DELETE_WINDOW,
						  (XtCallbackProc) FSB_cancelCB, (XtPointer) 0);
  XtManageChild(dialog);
  XtPopup(getShell(dialog), XtGrabNone);
}


void
FSB_cancelCB(w, client_data, cb)
Widget w;
XtPointer client_data;
XmAnyCallbackStruct *cb;
{
  XtDestroyWidget(w);
}

void
FSB_okCB(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer *call_data;
{
  char * dataFile;

  int reason = (int) client_data;
  XmFileSelectionBoxCallbackStruct *cbs =
	(XmFileSelectionBoxCallbackStruct *) call_data;

  dataFile = XrtGearCvtXmStringToString(cbs->value);

  switch(reason) {
  case OPEN_FILE:
	openTreeData(XrtOutliner1, dataFile);
	break;
  case SAVE_FILE_AS:
	saveTreeData(XrtOutliner1, dataFile);
	break;
  case SAVE_FILE:
	saveDataFile(XrtOutliner1, dataFile);
	break;
  default:  
	break;
  }
  XtDestroyWidget(w);
}

/*
 * Add/remove column CB's
 */

void
removeColumn(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer *call_data;
{
  XrtGearObject columnList = NULL;
  int position;

  if (XrtOutliner1 == NULL) {
	return;
  }

  XtVaGetValues(XrtOutliner1,
				XmNxrtGearColumnList, &columnList,
				NULL);
  
  position = XrtGearListGetItemCount(columnList);
  if (position > 1) {
	XtDestroyWidget( * (Widget*) XrtGearListGetItem(columnList, position-1));
  } else {
	printf("Can't destroy XrtColumn at position: %d\n",position);
  }  
}

void
addColumn(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer *call_data;
{
  XrtGearObject columnList = NULL;
  char name[80];
  int number;

  if (XrtOutliner1 == NULL) {
	return;
  }
  
  XtVaGetValues(XrtOutliner1,
				XmNxrtGearColumnList, &columnList,
				NULL);
  
  number = XrtGearListGetItemCount(columnList) + 1;
  sprintf(name, "Column %d", number);
  XtVaCreateWidget( "column",
					xmXrtColumnObjectClass, XrtOutliner1,
					XmNxrtGearLabel,
					XrtGearStringCreateCharString(name),
					XmNxrtGearLabelAlignment, XRTGEAR_ALIGN_BEGINNING,
					NULL);
}


/* main dialog CB's */

void
cancelCB(w, client_data, cb)
Widget w;
XtPointer client_data;
XmAnyCallbackStruct *cb;
{
  UxDestroyInterface( treeEd );
}

void
okCB(w, client_data, call_data)
Widget w;
XtPointer client_data;
XtPointer *call_data;
{
  Widget parent = (Widget) client_data;
  char *dataFile;
  
  XtVaGetValues(XrtOutliner1, XmNxrtGearTreeData, &dataFile, NULL);
  if (!dataFile || !*dataFile || (dataModified == True)) {
	popupFSB(w, (XtPointer)SAVE_FILE, NULL);
  } else {
    UxPeDepositValue( XrtPedit, treeEdSlot, UxQuoteWrap(dataFile));
	UxDestroyInterface( treeEd );
  }
}

void
helpCB(w, client_data, cb)
Widget w;
XtPointer client_data;
XmAnyCallbackStruct *cb;
{
#ifdef USEXRTTOOLS
  MtShowGearAboutDialog(UxGetWidget(treeEd), helpMsg);
#else
  printf("%s\n", CGETS_KL(TREE_MSG1));
  printf("---------------\n");
  printf("\n");
  printf("%s\n", CGETS_KL(TREE_MSG2));
  printf("%s\n", CGETS_KL(TREE_MSG3));
  printf("%s\n", CGETS_KL(TREE_MSG4));
  printf("%s\n", CGETS_KL(TREE_MSG5));
  printf("%s\n", CGETS_KL(TREE_MSG6));
  printf("\n");
  printf("%s\n", CGETS_KL(TREE_MSG7));
  printf("%s\n", CGETS_KL(TREE_MSG8));
  printf("%s\n", CGETS_KL(TREE_MSG9));
  printf("%s\n", CGETS_KL(TREE_MSG10));
  printf("%s\n", CGETS_KL(TREE_MSG11));
  printf("\n");
  printf("%s\n", CGETS_KL(TREE_MSG12));
  printf("%s\n", CGETS_KL(TREE_MSG13));
  printf("%s\n", CGETS_KL(TREE_MSG14));
  printf("%s\n", CGETS_KL(TREE_MSG15));
  printf("%s\n", CGETS_KL(TREE_MSG16));
  printf("%s\n", CGETS_KL(TREE_MSG17));
#endif 
}

static void  destroyTreeEdCB( UxWidget, UxClientData, UxCallbackArg )
	Widget		UxWidget;
	XtPointer	UxClientData, UxCallbackArg;

{
  treeEdIsUsed = False;
  UxPeRelease( XrtPedit, treeEd);
}

/*Entry point to tree editor form UIMX*/
swidget
UxPopupXrtTreeEd( pedit, resname, printname, swidgets, nswidgets)
    swidget  pedit;
    char     *resname;
    char     *printname;
    swidget  *swidgets;
    int      nswidgets;
{

    char *value, *raw_value;

    if ( treeEdIsUsed == False ) {
	treeEdIsUsed = True;
    } else {
	printf("Can't open Tree Editor.\nSorry, only one tree editor at a time is available\n");
	return pedit;
    }
    XrtPedit = pedit;

    /* The following UxPeReserve() requires treeEd. */
    /* So this code must be here. */
    treeEd = create_treeEd();

    /*
     *  Reserve this PE slot while we are editing tree
     */
    UxPeReserve( XrtPedit, resname, treeEd, &value, &treeEdSlot);

    /*
     *  If the slot has a value grunge it back into the file browser
     */
    raw_value = UxUnquoteWrap( value);
	openTreeData( XrtOutliner1, raw_value);

    /*
     * Display the tree Editor in all its glory!
     */
    UxPopupInterface( treeEd, no_grab);
	return (pedit);
}



/*******************************************************************************
       The 'build_' function creates all the widgets
       using the resource values specified in the Property Editor.
*******************************************************************************/


static swidget	create_treeEd()
{
    /* Creation of treeEd */
  treeEd = UxCreateDialogShell( "treeEd", NO_PARENT );
  UxPutX( treeEd, 0 ),
  UxPutY( treeEd, 0 ),
  UxPutTitle( treeEd, "XRT Tree Editor" ),
  UxCreateWidget( treeEd );

  UxAddCallback( treeEd, XmNdestroyCallback,
				 (XtCallbackProc) destroyTreeEdCB,
				 (XtPointer) NULL );


	form = XtVaCreateWidget( "form",
			xmFormWidgetClass,
			UxGetWidget(treeEd),
			NULL );

	/* Creation of XrtToolbar1 */
	XrtToolbar1 = XtVaCreateManagedWidget( "XrtToolbar1",
			xmXrtToolbarWidgetClass,
			form,
			XmNleftAttachment, XmATTACH_FORM,
			XmNtopAttachment, XmATTACH_FORM,
			XmNleftOffset, 10,
			XmNtopOffset, 10,
			NULL );


	/* Creation of XrtGLabel3 */
	XrtGLabel3 = XtVaCreateManagedWidget( "XrtGLabel3",
			xmXrtGLabelWidgetClass,
			XrtToolbar1,
			RES_CONVERT( XmNlabelString,
				     CGETS_KL(VIEW_MULTI_COLUMN) ),
			NULL );


	/* Creation of arrowButton1 */
	arrowButton1 = XtVaCreateManagedWidget( "arrowButton1",
			xmArrowButtonWidgetClass,
			XrtToolbar1,
			XmNarrowDirection, XmARROW_LEFT,
			XmNshadowThickness, 2,
			NULL );
	XtAddCallback( arrowButton1, XmNactivateCallback,
		(XtCallbackProc) removeColumn,
		(XtPointer) NULL );



	/* Creation of arrowButton2 */
	arrowButton2 = XtVaCreateManagedWidget( "arrowButton2",
			xmArrowButtonWidgetClass,
			XrtToolbar1,
			XmNarrowDirection, XmARROW_RIGHT,
			XmNshadowThickness, 2,
			NULL );
	XtAddCallback( arrowButton2, XmNactivateCallback,
		(XtCallbackProc) addColumn,
		(XtPointer) NULL );

	/* Creation of XrtOutliner1 */
	XrtOutliner1 = XtVaCreateManagedWidget( "XrtOutliner1",
			xmXrtOutlinerWidgetClass,
			form,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNleftAttachment, XmATTACH_FORM,
			XmNleftOffset, 10,
			XmNtopAttachment, XmATTACH_WIDGET,
			XmNwidth, 456,
			XmNrightAttachment, XmATTACH_FORM,
			XmNrightOffset, 10,
			XmNtopOffset, 10,
			XmNbottomOffset, 290,
			XmNheight, 235,
			XmNtopWidget, XrtToolbar1,
			XmNxrtGearColumnLabelsShow, TRUE,
			XmNxrtGearNodeMovePolicy, XRTGEAR_IMPORT_WITHIN_WIDGET,
			NULL );
	XtAddCallback( XrtOutliner1, XmNxrtGearSelectionCallback,
		(XtCallbackProc) selectItemCB,
		(XtPointer) NULL );
	XtAddCallback( XrtOutliner1, XmNxrtGearFolderStateCallback,
		(XtCallbackProc) folderStateCB,
		(XtPointer) NULL );
	XtAddCallback( XrtOutliner1, XmNxrtGearNodeMoveCallback,
		(XtCallbackProc) nodeMoveCB,
		(XtPointer) NULL );

	/* Creation of XrtColumn1 */
	XrtColumn1 = XtVaCreateWidget( "XrtColumn1",
			xmXrtColumnObjectClass,
			XrtOutliner1,
			XmNx, 0,
			XmNy, 0,
			RES_CONVERT( XmNxrtGearLabel, "Column 1" ),
			XmNxrtGearLabelAlignment, XRTGEAR_ALIGN_BEGINNING,
			NULL );

	/* Creation of XrtAligner1 */
	XrtAligner1 = XtVaCreateManagedWidget( "XrtAligner1",
			xmXrtAlignerWidgetClass,
			form,
			XmNleftAttachment, XmATTACH_FORM,
			XmNleftOffset, 10,
			XmNtopOffset, 30,
			XmNtopAttachment, XmATTACH_WIDGET,
			XmNtopWidget, XrtOutliner1,
			XmNrightAttachment, XmATTACH_OPPOSITE_WIDGET,
			XmNrightWidget, XrtOutliner1,
			XmNxrtGearMarginHeight, 2,
			NULL );

	/* Creation of XrtGLabel1 */
	XrtGLabel1 = XtVaCreateManagedWidget( "XrtGLabel1",
			xmXrtGLabelWidgetClass,
			XrtAligner1,
			RES_CONVERT( XmNlabelString, CGETS_KL(CREATE_ITEM) ),
			NULL );

#ifdef __hpux
	/* workaround for HP-UX XrtToggleButton problem */
	/* Use XrtPushButton instaed XrtToggleButton */
	/* ** caution ** the name of the widget is still XrtToggleB1 */
	XrtToggleB1 = XtVaCreateManagedWidget( "XrtToggleB1",
			xmXrtPushButtonWidgetClass,
			XrtAligner1,
			RES_CONVERT( XmNlabelString, CGETS_KL(FOLDER) ),
			RES_CONVERT( XmNxrtGearMask, "$XRTHOME/src/icons/folder_open.xpm" ),
			RES_CONVERT( XmNxrtGearLabelPixmap, "$XRTHOME/src/icons/folder_open.xpm" ),
			XmNxrtGearLabelType, XRTGEAR_TYPE_STRING_PIXMAP,
			XmNshadowThickness, 2,
			XmNuserData, (XtPointer)0,
			NULL );
	XtAddCallback( XrtToggleB1, XmNactivateCallback,
		(XtCallbackProc) toggleNodeOrFolderCB,
		(XtPointer) NULL );
#else
	/* Creation of XrtToggleB1 */
	XrtToggleB1 = XtVaCreateManagedWidget( "XrtToggleB1",
			xmXrtToggleButtonWidgetClass,
			XrtAligner1,
			RES_CONVERT( XmNlabelString, CGETS_KL(FOLDER) ),
			RES_CONVERT( XmNxrtGearIndicatorMaskList, "$XRTHOME/src/icons/folder_open.xpm, $XRTHOME/src/icons/item2.xpm" ),
			RES_CONVERT( XmNxrtGearIndicatorPixmapList, "$XRTHOME/src/icons/folder_open.xpm, $XRTHOME/src/icons/item2.xpm" ),
			XmNxrtGearIndicator, XRTGEAR_INDICATOR_PIXMAP,
			XmNshadowThickness, 2,
			NULL );
	XtAddCallback( XrtToggleB1, XmNxrtGearStateChangedCallback,
		(XtCallbackProc) toggleNodeOrFolderCB,
		(XtPointer) NULL );
#endif

	/* Creation of XrtGLabel2 */
	XrtGLabel2 = XtVaCreateManagedWidget( "XrtGLabel2",
			xmXrtGLabelWidgetClass,
			XrtAligner1,
			RES_CONVERT( XmNlabelString, CGETS_KL(SET_LABEL) ),
			NULL );


	/* Creation of textField1 */
	textField1 = XtVaCreateManagedWidget( "textField1",
			xmTextFieldWidgetClass,
			XrtAligner1,
			XmNcolumns, 45,
			NULL );
	XtAddCallback( textField1, XmNactivateCallback,
		(XtCallbackProc) modifyLabelCB,
		(XtPointer) NULL );

	/* Creation of rowColumn1 */
	rowColumn1 = XtVaCreateManagedWidget( "rowColumn1",
			xmRowColumnWidgetClass,
			form,
			XmNbottomAttachment, XmATTACH_NONE,
			XmNrightAttachment, XmATTACH_NONE,
			XmNleftAttachment, XmATTACH_FORM,
			XmNentryAlignment, XmALIGNMENT_CENTER,
			XmNtopAttachment, XmATTACH_WIDGET,
			XmNtopOffset, 30,
			XmNtopWidget, XrtAligner1,
			XmNwidth, 129,
			XmNleftOffset, 10,
			XmNorientation, XmHORIZONTAL,
			XmNpacking, XmPACK_COLUMN,
			NULL );


	/* Creation of XrtPushB1 */
	XrtPushB1 = XtVaCreateManagedWidget( "XrtPushB1",
			xmXrtPushButtonWidgetClass,
			rowColumn1,
			RES_CONVERT( XmNlabelString, CGETS_KL(ADD_ITEM) ),
			NULL );
	XtAddCallback( XrtPushB1, XmNactivateCallback,
		(XtCallbackProc) createNodeCB,
		(XtPointer) NULL );



	/* Creation of XrtPushB2 */
	XrtPushB2 = XtVaCreateManagedWidget( "XrtPushB2",
			xmXrtPushButtonWidgetClass,
			rowColumn1,
			RES_CONVERT( XmNlabelString, CGETS_KL(DELETE_ITEM) ),
			NULL );
	XtAddCallback( XrtPushB2, XmNactivateCallback,
		(XtCallbackProc) deleteNodeCB,
		(XtPointer) NULL );



	/* Creation of XrtPushB3 */
	XrtPushB3 = XtVaCreateManagedWidget( "XrtPushB3",
			xmXrtPushButtonWidgetClass,
			rowColumn1,
			RES_CONVERT( XmNlabelString, CGETS_KL(OPEN_DATA_FILE) ),
			NULL );
	XtAddCallback( XrtPushB3, XmNactivateCallback,
		(XtCallbackProc) popupFSB,
		(XtPointer) OPEN_FILE );



	/* Creation of XrtPushB4 */
	XrtPushB4 = XtVaCreateManagedWidget( "XrtPushB4",
			xmXrtPushButtonWidgetClass,
			rowColumn1,
			RES_CONVERT( XmNlabelString, CGETS_BALLOON(IN_SAVEAS) ),
			NULL );
	XtAddCallback( XrtPushB4, XmNactivateCallback,
		(XtCallbackProc) popupFSB,
		(XtPointer) SAVE_FILE_AS );



	/* Creation of separator1 */
	separator1 = XtVaCreateManagedWidget( "separator1",
			xmSeparatorWidgetClass,
			form,
			XmNwidth, 100,
			XmNheight, 20,
			XmNx, 0,
			XmNy, 0,
			XmNtopAttachment, XmATTACH_WIDGET,
			XmNtopWidget, rowColumn1,
			XmNrightAttachment, XmATTACH_FORM,
			XmNrightOffset,2,
			XmNleftAttachment, XmATTACH_FORM,
			XmNleftOffset,2,
			XmNtopOffset, 20,
			NULL );


	/* Creation of form1 */
	form1 = XtVaCreateManagedWidget( "form1",
			xmFormWidgetClass,
			form,
			XmNresizePolicy, XmRESIZE_NONE,
			XmNx, -1,
			XmNy, 540,
			XmNrightAttachment, XmATTACH_FORM,
			XmNrightOffset,2,
			XmNtopAttachment, XmATTACH_WIDGET,
			XmNtopOffset, 20,
			XmNtopWidget, separator1,
			XmNleftAttachment, XmATTACH_FORM,
			XmNleftOffset,2,
			NULL );


	/* Creation of XrtPushB5 */
	XrtPushB5 = XtVaCreateManagedWidget( "XrtPushB5",
			xmXrtPushButtonWidgetClass,
			form1,
			XmNx, 66,
			XmNy, -1,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNleftOffset, 20,
			XmNrightAttachment, XmATTACH_POSITION,
			XmNrightPosition, 32,
			XmNleftAttachment, XmATTACH_FORM,
			XmNtopAttachment, XmATTACH_FORM,
			RES_CONVERT( XmNlabelString, CGETS_COM(PB_OK) ),
			NULL );
	XtAddCallback( XrtPushB5, XmNactivateCallback,
		(XtCallbackProc) okCB,
		(XtPointer) NULL );



	/* Creation of XrtPushB6 */
	XrtPushB6 = XtVaCreateManagedWidget( "XrtPushB6",
			xmXrtPushButtonWidgetClass,
			form1,
			XmNx, 275,
			XmNy, -1,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNrightAttachment, XmATTACH_POSITION,
			XmNrightPosition, 64,
			XmNleftAttachment, XmATTACH_WIDGET,
			XmNleftOffset, 20,
			XmNleftWidget, XrtPushB5,
			XmNtopAttachment, XmATTACH_FORM,
			RES_CONVERT( XmNlabelString, CGETS_COM(PB_CANCEL) ),
			NULL );
	XtAddCallback( XrtPushB6, XmNactivateCallback,
		(XtCallbackProc) cancelCB,
		(XtPointer) NULL );



	/* Creation of XrtPushB7 */
	XrtPushB7 = XtVaCreateManagedWidget( "XrtPushB7",
			xmXrtPushButtonWidgetClass,
			form1,
			XmNx, 0,
			XmNy, 0,
			XmNleftAttachment, XmATTACH_WIDGET,
			XmNleftWidget, XrtPushB6,
			XmNrightAttachment, XmATTACH_FORM,
			XmNrightOffset, 20,
			XmNbottomAttachment, XmATTACH_FORM,
			XmNtopAttachment, XmATTACH_FORM,
			RES_CONVERT( XmNlabelString, CGETS_KL(PB_HELP) ),
			XmNleftOffset, 20,
			NULL );
	XtAddCallback( XrtPushB7, XmNactivateCallback,
		(XtCallbackProc) helpCB,
		(XtPointer) NULL );

	UxRealizeInterface( treeEd );

	return ( treeEd );
}

#endif	/* HASXRTGEAR*/

/*******************************************************************************
       END OF FILE
*******************************************************************************/
