/* ALL RIGHTS RESERVED
 *
 * This file is provided for demonstration and educational uses only.
 * Permission to use, copy, modify and distribute this file for
 * any purpose and without fee is hereby granted, provided that the
 * above copyright notice and this permission notice appear in all
 * copies, and that the name of KL Group not be used in advertising
 * or publicity pertaining to this material without the specific,
 * prior written permission of an authorized representative of
 * KL Group.
 *
 * KL GROUP MAKES NO REPRESENTATIONS AND EXTENDS NO WARRANTIES, EX-
 * PRESS OR IMPLIED, WITH RESPECT TO THE SOFTWARE, INCLUDING, BUT
 * NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
 * FITNESS FOR ANY PARTICULAR PURPOSE, AND THE WARRANTY AGAINST IN-
 * FRINGEMENT OF PATENTS OR OTHER INTELLECTUAL PROPERTY RIGHTS.  THE
 * SOFTWARE IS PROVIDED "AS IS", AND IN NO EVENT SHALL KL GROUP OR
 * ANY OF ITS AFFILIATES BE LIABLE FOR ANY DAMAGES, INCLUDING ANY
 * LOST PROFITS OR OTHER INCIDENTAL OR CONSEQUENTIAL DAMAGES
 * RELATING TO THE USE OF THIS SOFTWARE.
 *
 *
 * XRT products and UIM/X integration file
 *
 * $AstecId: Util.c,v 1.7 2003/01/29 04:23:37 kosaka Exp $
 *  SCCSID -- @(#)Util.c	1.4    97/02/20 KL Group Inc.
 */

/******************************************************************************
				Util.c
******************************************************************************/

#include <stdio.h>
#include <string.h>
#include <UxLib.h>
#include <vtypes.h>
#include <utype.h>
#include <Xm/Xm.h>
#ifdef	HASXRTSNAPSHOT
#include <Xm/XrtGear.h>
#endif
#include <X11/Xatom.h>

#include "Xrt_restable.h"
#include "cat_macros.h"

XrtStringInfo	**xrt_string_table = NULL;
int		n_string_info = 0;
static char	xrt_debug_file[80];
static FILE	*fd_error_log = NULL;


/* Only SunOS 4.1 doesn't define memmove */
#if sun4 && !SOLARIS

void *
memmove (s1, s2, n)
char * s1;
char * s2;
int n;
{
  return((void *) memcpy(s1, s2, n));
}

#endif


XrtStringInfo *
FindXrtStringInfo(func, identifier, sw)
int		(*func)();
int		identifier;
swidget	sw;
{
	XrtStringInfo	*info;
	int		i;
	
	if (xrt_string_table) {
		for (i = 0; xrt_string_table[i]; i++) {
			info = xrt_string_table[i];
			if ((info->func == func) &&
				(info->identifier == identifier) &&
				(info->sw == sw))
			{
				return(info);
			}
		}
	}
	return(NULL);	
}

void
RegisterXrtStringInfo(func, identifier, sw, string)
int		(*func)();
int		identifier;
swidget	sw;
char	*string;
{
	XrtStringInfo	*info;
	int		n;
	
	n = n_string_info;
	if (xrt_string_table) {
		if ((info = FindXrtStringInfo(func, identifier, sw)) != NULL) {
			if (info->string && strcmp(info->string, string)) {
				free(info->string);
				info->string = XtNewString(string);
			}
			return;
		}
		xrt_string_table = (XrtStringInfo **)
			realloc(xrt_string_table, (n + 2) * sizeof(XrtStringInfo *));
	}
	else {
		xrt_string_table = (XrtStringInfo **)
			calloc(n + 2, sizeof(XrtStringInfo *));
	}
	
	info = (XrtStringInfo *) calloc (1, sizeof(XrtStringInfo));
	info->func = func;
	info->identifier = identifier;
	info->sw = sw;
	info->string = XtNewString(string);
	xrt_string_table[n++] = info;
	xrt_string_table[n] = NULL;
	n_string_info = n;
}

char *
XrtEscapeQuotes(str)
char *str;
{
	/* Process embedded quotes in a string.  Also allocate enough space. */

	int	str_length = 0;
	char 	*t, *s, *string;

	if (!str || !str[0]) {
		string = XtNewString("");
		return(string);
	}
	
	for (t = str; *t;  t++) {
		str_length++;
		if (*t == '\"') {
			str_length++;
		}
	}

	s = string = (char *) XtMalloc (str_length + 1);
	for (t = str; *t;  t++) {
		if (*t == '\"') {
			*s++ = '\\';
		}
		*s++ = *t;
	}
	
	*s = '\0';
	return(string);
}

/* rb: remove this function from XrtGph and Xrt3d _xtypes.c*/
void
XrtInitCommonXtypes()
{
	static Boolean	done = FALSE;

	if (done) {
		return;
	}

	done = TRUE;
}

/******************************************************************************
                Debug Subroutines
******************************************************************************/

#if defined(HASXRTGAUGE) || defined(DEBUG)

xrt_debug_init()
{
#ifdef DEBUG
	if( fd_error_log != NULL) {
		return;
	}
	sprintf(xrt_debug_file, "/tmp/xrt_debug_log.%d", getpid());
	if ((fd_error_log = fopen(xrt_debug_file, "w")) == NULL) {
		fprintf(stderr, "Can't create %s\n", xrt_debug_file);
		exit(1);
	}
	printf("Logging xrt debugging information to %s\n", xrt_debug_file);
#endif /* DEBUG */
}

xrt_debug_enter(func_name)
char *func_name;
{
#ifdef DEBUG
	if( fd_error_log == NULL) {
		xrt_debug_init();
	}
	fprintf(fd_error_log, "Enter: %25s()\n", func_name);
	fflush(fd_error_log);
#endif /* DEBUG */
}

xrt_debug_leave(func_name)
char *func_name;
{
#ifdef DEBUG
	if( fd_error_log == NULL) {
		xrt_debug_init();
	}
	fprintf(fd_error_log, "Leave: %25s()\n", func_name);
	fflush(fd_error_log);
#endif /* DEBUG */
}

xrt_debug_print_string(message, value)
char *message;
char *value;
{
#ifdef DEBUG
	if( fd_error_log == NULL) {
		xrt_debug_init();
	}
	if (message) {
		fprintf(fd_error_log, "%s --> %s\n", message, value);
	}
	else {
		fprintf(fd_error_log, "%s\n", value);
	}
	fflush(fd_error_log);
#endif /* DEBUG */
}

xrt_debug_print_int(message, value)
char *message;
int	 value;
{
#ifdef DEBUG
	if( fd_error_log == NULL) {
		xrt_debug_init();
	}
	fprintf(fd_error_log, "%s %d\n", message, value);
	fflush(fd_error_log);
#endif /* DEBUG */
}

xrt_debug_print_float(message, value)
char 	*message;
float	 value;
{
#ifdef DEBUG
	if( fd_error_log == NULL) {
		xrt_debug_init();
	}
	fprintf(fd_error_log, "%s %f\n", message, value);
	fflush(fd_error_log);
#endif /* DEBUG */
}
#endif /* defined(HASXRTGAUGE) || defined(DEBUG) */

#ifdef DESIGN_TIME
/*
 * This callback function activates Widget Snapshot Utility in UIM/X
 *
 * if $(XRTHOME)/src/gear/tools/libtools.a exists then
 * HASXRTTOOLS is defined and Print Dialog will popup for Snapshot
 * otherwise Snapshot output will be sent to .ps file.
 * 
 */  

void XrtWidgetSnapshotCBProc()
{
#ifdef	HASXRTSNAPSHOT
#include "mtgear.h"
#include "snapshot.h"
#include "motif_tools.h"


  Widget toplevel = UxGetWidget(UxGetSelectedIface());

  if(! toplevel) {
	fprintf(stdout, "%s", CGETS_KL(SNAPSHOT_WARNING));
	return;
  }

  /* use Print Popup Dialog from XRT/gear libtools.a */
  MtCreatePopupShell(MtGetShell(UxTopLevel),
					 (MtCreatePopupChildProc) MtGearCreatePrintWidgetPage,
					 "Print Application", (XtPointer) MtGetShell(toplevel));
#endif	/* HASXRTSNAPSHO */
}
#endif /* DESIGN_TIME */

#ifdef DESIGN_TIME
double UxXrtGET_double ( sw, resbp, name )
        swidget  sw;
        binptr   resbp;
        char     *name;
{
        return (double)UxGET_float(sw,resbp,name);
}

int UxXrtPUT_double (sw, resbp, name, value)
        swidget  sw;
        binptr   resbp;
        char     *name;
        double   value;
{
        float f;
        f=(float)value;
        return UxPUT_int(sw, resbp, name, *(int *)&f);
}

/* for interpreter bug */
#ifdef hp700v9
#ifdef HASXRTFIELD
XtArgVal UxXrtFldDoubleToArgVal(stub, val)
     int stub;
     double val;
{
  
        return XrtFldDoubleToArgVal(val);
}
#endif /* HASXRTFIELD */

#ifdef HASXRT3D
XtArgVal UxXrt3dDoubleToArgVal(stub, val)
     int stub;
     double val;
{
  
        return Xrt3dDoubleToArgVal(val);
}
#endif /* HASXRT3D */

#ifdef HASXRTGRAPH
XtArgVal UxXrtFloatToArgVal(stub, val)
     int stub;
     double val;
{
  
        return XrtFloatToArgVal((float) val);
}
#endif /* HASXRTGRAPH */

#ifdef HASXRTGAUGE
XtArgVal UxXrtGaDoubleToArgVal(stub, val)
     int stub;
     double val;
{

        return XrtGaDoubleToArgVal(val);
}
#endif /* HASXRTGAUGE */

#endif /* hp700v9 */

#endif /* DESIGN_TIME */

